@@ -24,6 +24,8 @@
 #include "config.h"
 #endif
 
+/* this is for lrint */
+#define _ISOC99_SOURCE
 #include <math.h>
 #include <string.h>
 
@@ -99,19 +101,10 @@ static void	cb_ui_event			(BonoboUIComponent *comp,
 						 gpointer           data);
 static void	cb_theme_change                (GtkIconTheme *icon_theme,
 						gpointer      data);
+static void	cb_stop_scroll_events		(GtkWidget *widget,
+						 GdkEvent  *event);
 
 static PanelAppletClass *parent_class = NULL;
-static struct {
-  gchar *filename;
-  GdkPixbuf *pixbuf;
-} pix[] = {
-  { "stock_volume-mute", NULL },
-  { "stock_volume-0",    NULL },
-  { "stock_volume-min",  NULL },
-  { "stock_volume-med",  NULL },
-  { "stock_volume-max",  NULL },
-  { NULL, NULL }
-};
 
 GType
 gnome_volume_applet_get_type (void)
@@ -144,24 +137,32 @@ gnome_volume_applet_get_type (void)
 static void
 init_pixbufs (GnomeVolumeApplet *applet)
 {
+  static const gchar *pix_filenames[] = {
+    "stock_volume-mute",
+    "stock_volume-0",
+    "stock_volume-min",
+    "stock_volume-med",
+    "stock_volume-max",
+    NULL
+  };
   gint n;
   
-  for (n = 0; pix[n].filename != NULL; n++) {
-    if (pix[n].pixbuf)
-      g_object_unref (pix[n].pixbuf);
+  for (n = 0; pix_filenames[n] != NULL; n++) {
+    if (applet->pix[n])
+      g_object_unref (applet->pix[n]);
     
-    pix[n].pixbuf = gtk_icon_theme_load_icon (
+    applet->pix[n] = gtk_icon_theme_load_icon (
       	    applet->icon_theme,
-      	    pix[n].filename,
+      	    pix_filenames[n],
       	    panel_applet_get_size (&applet->parent),
       	    0,
       	    NULL);
     if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
       GdkPixbuf *temp;
 
-      temp = gdk_pixbuf_flip (pix[n].pixbuf, TRUE);
-      g_object_unref (G_OBJECT (pix[n].pixbuf));
-      pix[n].pixbuf = temp;
+      temp = gdk_pixbuf_flip (applet->pix[n], TRUE);
+      g_object_unref (G_OBJECT (applet->pix[n]));
+      applet->pix[n] = temp;
     }
   }
 }
@@ -202,6 +203,7 @@ gnome_volume_applet_init (GnomeVolumeApplet *applet)
   applet->state = -1;
   applet->prefs = NULL;
   applet->icon_theme = gtk_icon_theme_get_default ();
+  applet->dock = NULL;
 
   /* init pixbufs */
   init_pixbufs (applet);
@@ -221,6 +223,11 @@ gnome_volume_applet_init (GnomeVolumeApplet *applet)
   gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET (applet),
 			_("Volume Control"), NULL);
 
+  /* prevent scroll events from reaching the tooltip */
+  g_signal_connect (G_OBJECT (applet),
+		    "event-after", G_CALLBACK (cb_stop_scroll_events),
+		    NULL);
+
   /* handle icon theme changes */
   g_signal_connect (G_OBJECT (applet->icon_theme),
 		    "changed", G_CALLBACK (cb_theme_change),
@@ -260,7 +267,12 @@ select_tracks (GstElement *element,
   active_track_name_list = NULL;
 
   if (reset_state) {
+#ifdef HAVE_GST10
+    gst_element_set_state (element, GST_STATE_READY);
+    if (gst_element_get_state(element, NULL, NULL, -1) != GST_STATE_CHANGE_SUCCESS)
+#else
     if (gst_element_set_state (element, GST_STATE_READY) != GST_STATE_SUCCESS)
+#endif
       return NULL;
   }
 
@@ -448,6 +460,7 @@ static void
 gnome_volume_applet_dispose (GObject *object)
 {
   GnomeVolumeApplet *applet = GNOME_VOLUME_APPLET (object);
+  gint n;
 
   gnome_volume_applet_popdown_dock (applet);
 
@@ -482,6 +495,13 @@ gnome_volume_applet_dispose (GObject *object)
     applet->timeout = 0;
   }
 
+  for (n = 0; n < 5; n++) {
+    if (applet->pix[n] != NULL) {
+      g_object_unref (G_OBJECT (applet->pix[n]));
+      applet->pix[n] = NULL;
+    }
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -831,6 +851,9 @@ gnome_volume_applet_key (GtkWidget   *widget,
       GtkAdjustment *adj = gtk_range_get_adjustment (applet->dock->scale);
       gdouble volume = adj->value, increment;
 
+      if (event->state != 0)
+        break;
+
       if (event->keyval == GDK_Up || event->keyval == GDK_Down)
         increment = adj->step_increment;
       else
@@ -876,7 +899,7 @@ gnome_volume_applet_orientation	(PanelApplet *_applet,
      *    thinks that the child of the applet (image) is us (dock),
      *    which is not the case.
      */
-    gtk_widget_destroy (GTK_WIDGET (applet->dock));
+    gtk_widget_unparent (GTK_WIDGET (applet->dock));
   }
   dock = gnome_volume_applet_dock_new (IS_PANEL_HORIZONTAL (orientation) ?
       GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL);
@@ -1056,9 +1079,9 @@ gnome_volume_applet_refresh (GnomeVolumeApplet *applet,
 
   if (did_change) {
     if (mute) {
-      pixbuf = pix[0].pixbuf;
+      pixbuf = applet->pix[0];
     } else {
-      pixbuf = pix[n].pixbuf;
+      pixbuf = applet->pix[n];
     }
 
     gtk_image_set_from_pixbuf (applet->image, pixbuf);
@@ -1158,8 +1181,12 @@ cb_gconf (GConfClient *client,
 
           if (new_element != old_element) {
             /* change element */
-            if (gst_element_set_state (item->data,
-				       GST_STATE_READY) != GST_STATE_SUCCESS)
+#ifdef HAVE_GST10
+            gst_element_set_state (item->data, GST_STATE_READY);
+            if (gst_element_get_state (item->data, NULL, NULL, -1) != GST_STATE_CHANGE_SUCCESS)
+#else
+            if (gst_element_set_state (item->data, GST_STATE_READY) != GST_STATE_SUCCESS)
+#endif
               continue;
 	    
             /* save */
@@ -1255,17 +1282,27 @@ cb_verb (BonoboUIComponent *uic,
     const gchar *authors[] = { "Ronald Bultje <rbultje@ronald.bitfreak.net>",
 			     NULL };
 
+    char *comments = g_strdup_printf ("%s\n\n%s",
+		    _("Volume control for your GNOME Panel."),
+#ifdef HAVE_GST10
+		    _("Using GStreamer 0.10.")
+#else
+		    _("Using GStreamer 0.8.")
+#endif
+		    );
+
     gtk_show_about_dialog (NULL,
 		"name",		_("Volume Applet"),
 		"version",	VERSION,
 		"copyright",	"\xC2\xA9 2004 Ronald Bultje",
-		"comments",	_("A GNOME/GStreamer-based volume control "
-				  "applet"),
+		"comments",	comments,
 		"authors",	authors,
 		"translator-credits",	_("translator-credits"),
 		"logo-icon-name",	"volume-knob",
 		NULL);
 
+    g_free (comments);
+
   } else if (!strcmp (verbname, "Pref")) {
     if (!applet->mixer) {
       show_no_mixer_dialog (applet);
@@ -1318,3 +1355,16 @@ cb_theme_change (GtkIconTheme *icon_theme,
   init_pixbufs (applet);
   gnome_volume_applet_refresh (applet, TRUE);
 }
+
+/*
+ * Block the tooltips event-after handler on scroll events.
+ */
+
+static void
+cb_stop_scroll_events (GtkWidget *widget,
+		       GdkEvent  *event)
+{
+  if (event->type == GDK_SCROLL)
+    g_signal_stop_emission_by_name (widget, "event-after");
+}
+
